<?php
namespace Ep\Gath\DataStore;

use Ep\Gath\Gath;
use Ep\Gath\Plugin\GathPluginBridge;
use Ep\Gath\Plugin\Products;
use WC_Product_Data_Store_CPT;

/**
 * Class ProductDataStore
 *
 * @package Ep\Gath\DataStore
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/gath
 */
class ProductDataStore extends WC_Product_Data_Store_CPT
{
    use GathPluginBridge;

    /**
     * Read product data. Can be overridden by child classes to load other props.
     *
     * @param WC_Product $product Product object.
     * @since 3.0.0
     */
    protected function read_product_data( &$product ) {
        $id = $product->get_id();
        $gathPrices = $this->getGathPrices();
        $gathProducts = $this->getGathProducts();
        $gathStock = $this->getGathStock();

        $productResult = $gathProducts->getRawResult($id);

        $product->set_props(
            array(
                'sku'                => $productResult['sku'],
                'regular_price'      => $gathPrices->get_post_metadata(null, $id, '_regular_price'),
                'sale_price'         => $gathPrices->get_post_metadata(null, $id, '_sale_price'),
                'price'              => $gathPrices->get_post_metadata(null, $id, '_price'),
                'date_on_sale_from'  => '',
                'date_on_sale_to'    => '',
                'total_sales'        => '',
                'tax_status'         => 'taxable',
                'tax_class'          => $productResult['tax_class'] ?? ($productResult['tax_classHR'] ?? ''),
                'manage_stock'       => 'yes',
                'stock_quantity'     => $gathStock->get_post_metadata(null, $id, '_stock'),
                'stock_status'       => $gathStock->get_post_metadata(null, $id, '_stock_status'),
                'backorders'         => 'no',
                'sold_individually'  => 'no',
                'weight'             => '',
                'length'             => '',
                'width'              => '',
                'height'             => '',
                'upsell_ids'         => '',
                'cross_sell_ids'     => '',
                'purchase_note'      => '',
                'default_attributes' => '',
                'category_ids'       => [],
                'tag_ids'            => [],
                'shipping_class_id'  => intval($productResult['shippingclassid']),
                'virtual'            => 'no',
                'downloadable'       => 'no',
                'gallery_image_ids'  => array_filter( explode( ',', $gathProducts->get_post_metadata(null, $id, '_product_image_gallery'))),
                'download_limit'     => '',
                'download_expiry'    => '',
                'image_id'           => get_post_thumbnail_id( $id ),
            )
        );
    }

    /**
     * Read attributes from post meta.
     *
     * @param \WC_Product $product Product object.
     */
    protected function read_attributes(&$product)
    {
        if ($product->get_id() < Products::ID_OFFSET) {
            return parent::read_attributes($product);
        }

        $gath = $this->getGathProducts();

        $result = $gath->getRawResult($product->get_id());
        $attributes_key = array_key_exists('VarAttributes', $result) ? 'VarAttributes' : 'Attributes';

        if (isset($result[$attributes_key]) && is_array($result[$attributes_key]) && !empty($result[$attributes_key])) {
            /** @var \WC_Product_Attribute[] $attributes */
            $attributes = [];

            foreach ($result[$attributes_key] as $i => $attr) {
                $id = $attr['id'];

                if (!isset($attributes[$id])) {
                    $attribute = new \WC_Product_Attribute();
                    $attribute->set_id(0);
                    $attribute->set_name($attr['name']);
                    $attribute->set_options([$attr['value']]);
                    $attribute->set_position($i + 1);
                    $attribute->set_visible(1);
                    $attribute->set_variation($product->get_type() === 'variable' ? 1 : 0);
                    $attributes[$id] = $attribute;
                } else {
                    $options = $attributes[$id]->get_options();
                    $options[] = $attr['value'];
                    $attributes[$id]->set_options($options);
                }
            }

            $product->set_attributes(array_values($attributes));
        }
    }
}