<?php
namespace Ep\Gath\Moto;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Zend\Stratigility\MiddlewarePipe;
use Zend\Stratigility\MiddlewarePipeInterface;

/**
 * Class MotoMiddleware
 *
 * @package Ep\Gath\Moto
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class MotoMiddleware implements MiddlewarePipeInterface
{
    /** @var MiddlewarePipeInterface */
    protected $pipeline;
    /** @var SelectedMotorcycle */
    protected $selectedMotorcycle;

    /**
     * MotoMiddleware constructor.
     *
     * @param SelectedMotorcycle $selectedMotorcycle
     */
    public function __construct(SelectedMotorcycle $selectedMotorcycle)
    {
        $this->pipeline = new MiddlewarePipe();
        $this->selectedMotorcycle = $selectedMotorcycle;
    }

    /**
     * Process an incoming server request and return a response, optionally delegating
     * response creation to a handler.
     *
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     *
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        return $this->pipeline->process($request, $handler);
    }

    public function pipe(MiddlewareInterface $middleware): void
    {
        $this->pipeline->pipe($middleware);
    }

    /**
     * Handle the request and return a response.
     *
     * @param ServerRequestInterface $request
     *
     * @return ResponseInterface
     */
    public function handle(ServerRequestInterface $request): ResponseInterface
    {
        return $this->pipeline->handle($request);
    }
}