<?php
namespace Ep\Gath\Moto;

use Dflydev\FigCookies\Cookies;
use Ep\Gath\Gath;
use Ep\Middleware\BaseUrl;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;

/**
 * Class VehicleSlug
 *
 * @package Gath\Moto\Middleware
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/gath-moto
 */
class MotorcycleSlug implements MiddlewareInterface
{
    /** @var Gath */
    private $gath;
    /** @var SelectedMotorcycle */
    private $motorcycle;

    /**
     * MotorcycleSlug constructor.
     *
     * @param Gath $gath
     * @param SelectedMotorcycle $motorcycle
     */
    public function __construct(
        Gath $gath,
        SelectedMotorcycle $motorcycle
    ) {
        $this->gath = $gath;
        $this->motorcycle = $motorcycle;
    }

    /**
     * Process an incoming server request and return a response, optionally delegating
     * response creation to a handler.
     *
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     *
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        $uri = $request->getUri();
        $path = $uri->getPath();

        if (preg_match('#^/(\d+)(/|$)#', $path, $matches)) {
            $motorcycleId = intval($matches[1]);
            $motorcycle = $this->gath->getMotorcycle($motorcycleId);

            if ($motorcycle !== null) {
                $this->motorcycle->set($motorcycle);
            }

            $uri = $uri->withPath(
                preg_replace('#^/\d+#', '', $uri->getPath()) ?: '/'
            );

            $baseUri = $request->getAttribute(BaseUrl::BASE_URL);
            $replace = '/' . ltrim(sprintf('%s/%d', $baseUri, $motorcycleId), '/');

            $_SERVER['REQUEST_URI'] = preg_replace('#^' . $replace . '#', $baseUri, $_SERVER['REQUEST_URI']);
            $request = $request->withUri($uri);
        } else {
            $cookies = Cookies::fromRequest($request);

            if ($cookies->has('_moto_id')) {
                $motorcycleId = intval($cookies->get('_moto_id')->getValue());
                $motorcycle = $this->gath->getMotorcycle($motorcycleId);

                if ($motorcycle !== null) {
                    $this->motorcycle->set($motorcycle);
                }
            }
        }

        return $handler->handle($request);
    }
}