<?php
namespace Ep\Gath\Plugin;

use Ep\Gath\Gath;
use Ep\Gath\Moto\SelectedMotorcycle;
use Ep\Wp\Plugin\PluginInterface;
use WP_Post;
use WP_Term;

/**
 * Class MotoProducts
 *
 * @package Ep\Gath\Plugin
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/gath
 */
class MotoProducts implements PluginInterface
{
    /** @var  */
    protected $gath;
    /** @var SelectedMotorcycle */
    protected $selected;

    /**
     * MotoProducts constructor.
     *
     * @param Gath $gath
     * @param SelectedMotorcycle $selected
     */
    public function __construct(Gath $gath, SelectedMotorcycle $selected)
    {
        $this->selected = $selected;
    }

    /**
     * Dodaje akcije i filtere u WordPress.
     *
     * @return void
     */
    public function __invoke(): void
    {
//        add_filter('pre_post_link', [$this, 'pre_post_link'], 10, 1);
        add_filter('pre_term_link', [$this, 'pre_term_link'], 10, 2);
        add_filter('post_type_link', [$this, 'post_type_link'], 10, 2);
        add_filter('posts_pre_query', [$this, 'posts_pre_query'], 5, 2);
    }

    public function pre_post_link($permalink)
    {

        return $permalink;
    }

    public function pre_term_link($permalink, WP_Term $term)
    {
        if ('product_cat' === $term->taxonomy && $this->selected->getId() > 0) {
            return $this->selected->getId() . '/' . $permalink;
        }

        return $permalink;
    }

    public function post_type_link($permalink, WP_Post $post)
    {
        if ('product' === $post->post_type) {
            if ($motoId = $this->selected->getId()) {
                return rtrim(WP_HOME, '/') . '/' . $motoId . '/' . ltrim(preg_replace('#' . WP_HOME . '#', '', $permalink), '/');
            }
        }

        return $permalink;
    }

    public function posts_pre_query($null, \WP_Query $query)
    {
        if (!$query->is_main_query()) {
            return $null;
        }

        $category = get_queried_object();

        if (!$category instanceof WP_Term || $category->taxonomy !== 'product_cat') {
            return $null;
        }

        if ($genericArticleIds = get_term_meta($category->term_id, 'genericArticleMotoIds', true)) {
            $query->set('moto_genids', array_map('absint', explode(';', $genericArticleIds)));
            $query->set('moto_id', $this->selected->getId());
        }

        return $null;
    }
}